
#include "config.h"

#include <string.h>

#include "launcher.h"

#define VIEW_DESKTOP_ICON_KEY "desktop-icon"

/// declarations

static GtkWidget*
create_icon_widget (KibaDesktopIcon *icon);

/// size allocation

static void
icon_view_place (KibaLauncherIconGroup *group)
{
	GtkWidget *group_widget = GTK_WIDGET(group->object);
	GtkWidget *win, *dock;
	gint w, h, x, y;
	const KibaLayoutData *d;
	
	dock = group_widget->parent;
	win = dock->parent;
	
	if (!KIBA_IS_WIN(win)) {
		return;
	} if (!KIBA_IS_ICON_VIEW_WIN(group->view_win)) {
		return;
	}
	
	d = &kiba_layout_data[KIBA_DOCK(KIBA_WIN_CHILD(win))->d.position];
	
	struct {
		gint  *x, *y;
		gint  *w, *h;
		gint  alignment;
	} magic[2] = {
		{ &x, &y , &w, &h, d->valignment},
		{ &y, &x , &h, &w, d->halignment}
	};
	w = group->view_win->win->allocation.width;
	h = group->view_win->win->allocation.height;
	x = group_widget->allocation.x + GTK_WIDGET(win)->allocation.x;
	y = group_widget->allocation.y + GTK_WIDGET(win)->allocation.y;
	
	*magic[d->orientation].x -= (*magic[d->orientation].w - group_widget->allocation.width)/2;
	*magic[d->orientation].y -= (magic[d->orientation].alignment == 0 ?
			- group_widget->allocation.height : *magic[d->orientation].h);
	
	gdk_window_move (group->view_win->win->window, x, y);
}

/// signal handler

static void
icon_view_size_allocate_cb (GtkWidget *widget,
		GtkAllocation *allocation,
		gpointer data)
{
	icon_view_place (
		(KibaLauncherIconGroup*) data);
}

static void
icon_view_child_activated_cb (KibaIconView *icon_view,
		GtkWidget *child,
		gpointer user_data)
{
	KibaLauncherIconGroup *group = user_data;
	KibaDesktopIcon *desktop_icon =
		g_object_get_data (G_OBJECT(child), VIEW_DESKTOP_ICON_KEY);
	
	if (desktop_icon->entry.exec != NULL) {
		kiba_object_launch (group->object, desktop_icon->entry.exec);
	}
}

static void
icon_view_child_selected_cb (KibaIconView *icon_view,
		GtkWidget *child,
		gpointer user_data)
{
	KibaLauncherIconGroup *group = user_data;
	kiba_launcher_select_icon(group->launcher, group,
		g_object_get_data (G_OBJECT(child), VIEW_DESKTOP_ICON_KEY));
}

static void
drag_delete_cb (GtkWidget *widget,
		GdkDragContext *drag_context,
		gpointer user_data)
{
	KibaDesktopIcon *icon = user_data;
	kiba_launcher_remove (
		g_object_get_data(G_OBJECT(icon), ICON_LAUNCHER_KEY), icon);
}

static void
drag_data_get_cb (GtkWidget *widget,
		GdkDragContext *drag_context,
		GtkSelectionData *data,
		guint info,
		guint time,
		gpointer user_data)
{
	KibaDesktopIcon *icon = user_data;
	gtk_selection_data_set(data,
		GDK_SELECTION_TYPE_STRING,
		8,	// 8 bits per character.
		(guchar *)icon->entry.uri,
		strlen(icon->entry.uri));
}

static void
drag_received_cb (GtkWidget *widget,
		GdkDragContext *drag_context,
		gint x,
		gint y,
		GtkSelectionData *selection_data,
		guint info,
		guint time,
		gpointer user_data)
{
	KibaLauncherIconGroup *group = user_data;
	
	kiba_launcher_drag_receive_cb (
		group->launcher,
		group,
		drag_context,
		(gchar *) selection_data->data);
	
	gtk_drag_finish (drag_context, TRUE, FALSE, time);
}

void
icon_view_launcher_icon_added_cb(KibaLauncher *launcher,
		KibaDesktopIcon *desktop_icon,
		gpointer user_data)
{
	KibaLauncherIconGroup *group = user_data;
	if (KIBA_IS_ICON_VIEW_WIN(group->view_win)) {
		GtkWidget *box = create_icon_widget(desktop_icon);
		kiba_icon_view_win_add (group->view_win, box);
	}
}

void
icon_view_launcher_icon_removed_cb(KibaLauncher *launcher,
		KibaDesktopIcon *desktop_icon,
		gpointer user_data)
{
	KibaLauncherIconGroup *group = user_data;
	GList *l, *childs;
	
	childs = gtk_container_get_children(
		GTK_CONTAINER(group->view_win->icon_view));

	for (l = childs; l != NULL; l = childs) {
		GtkWidget *child = l->data;
		
		childs = g_list_remove(childs, l->data);
		
		if (desktop_icon ==
		    g_object_get_data (G_OBJECT(child), VIEW_DESKTOP_ICON_KEY)) {
			gtk_container_remove(
				GTK_CONTAINER(group->view_win->icon_view),
				child);
		}
	}
	g_list_free(childs);
}

void
icon_view_launcher_icon_selected_cb(KibaLauncher *launcher,
		KibaDesktopIcon *desktop_icon,
		gpointer user_data)
{
	KibaLauncherIconGroup *group = user_data;
	GList *l, *childs;
	
	childs = gtk_container_get_children(
		GTK_CONTAINER(group->view_win->icon_view));

	for (l = childs; l != NULL; l = childs) {
		GtkWidget *child = l->data;
		
		childs = g_list_remove(childs, l->data);
		
		if (desktop_icon ==
		    g_object_get_data (G_OBJECT(child), VIEW_DESKTOP_ICON_KEY)) {
			kiba_icon_view_select_child (
				KIBA_ICON_VIEW(group->view_win->icon_view),
				child);
		}
	}
	g_list_free(childs);
}

static void
icon_view_win_destroy_cb (GtkWidget *widget,
		gpointer user_data)
{
	KibaLauncherIconGroup *group = user_data;
	
	g_signal_handlers_disconnect_by_func (G_OBJECT(group->object),
		  G_CALLBACK(icon_view_size_allocate_cb), group);

	group->view_win = NULL;
}

/// helper

static GtkWidget*
create_icon_widget (KibaDesktopIcon *icon)
{
	static const GtkTargetEntry targets[] = {
		{ "text/uri-list", 0, 0 }
	};
	
	GtkWidget *image;
	GtkWidget *box;
	GtkWidget *label;
	char *markup;
	
	box = gtk_vbox_new(FALSE, 0);
	
	// add a image displaying the icon
	image = kiba_image_new ();
	kiba_image_set_icon(KIBA_IMAGE(image),
		icon->entry.icon,
		38, 38);
	kiba_image_set_size (KIBA_IMAGE(image), 38, 38);
	gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);
	
	// add a markup label for icon description
	label = gtk_label_new(icon->entry.name);
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
	gtk_label_set_ellipsize (GTK_LABEL(label), PANGO_ELLIPSIZE_END);
	markup = g_markup_printf_escaped ("<span size=\"small\">%s</span>",
			GTK_LABEL (label)->text);
	gtk_label_set_markup (GTK_LABEL (label), markup);
	g_free (markup);
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
	
	// enable drag and drop
	gtk_drag_source_set (image,
		GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
		targets, G_N_ELEMENTS (targets),
		GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_MOVE);
	
	// connect to widget signals
	g_signal_connect (GTK_OBJECT(image), "drag-data-get",
		G_CALLBACK(drag_data_get_cb), icon);
	g_signal_connect (GTK_OBJECT(image), "drag-data-delete",
		G_CALLBACK(drag_delete_cb), icon);
	
	// remember the icon on the box
	g_object_set_data (G_OBJECT(box),
		VIEW_DESKTOP_ICON_KEY, icon);
	
	gtk_widget_show (box);
	gtk_widget_show (image);
	
	return box;
}

static KibaIconViewWin*
create_icon_view (KibaLauncherIconGroup *group)
{
	static const GtkTargetEntry targets[] = {
		{ "text/uri-list", 0, 0 }
	};
	
	KibaIconViewWin *view_win;
	GtkWidget *box;
	GList *l;
	
	// create the window
	view_win = kiba_icon_view_win_new(
		KIBA_ICON_VIEW_BOX_POSITION_RIGHT, 4, 3, FALSE, FALSE, TRUE);
	gtk_window_set_type_hint (GTK_WINDOW(view_win->win),
		GDK_WINDOW_TYPE_HINT_DIALOG);
	gtk_window_stick (GTK_WINDOW(view_win->win));
	gtk_window_set_role (GTK_WINDOW(view_win->win),
		LAUNCHER_WINDOW_ROLE);
	
	// create a widget for each icon and add it to the view
	for (l = group->icons; l != NULL; l = l->next) {
		KibaDesktopIcon *icon = l->data;
		box = create_icon_widget(icon);
		kiba_icon_view_win_add (view_win, box);
	}
	
	// enable drag and drop
	gtk_drag_dest_set (view_win->icon_view,
		GTK_DEST_DEFAULT_ALL,
		targets, G_N_ELEMENTS (targets),
		GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_MOVE);
	gtk_drag_dest_add_uri_targets (view_win->icon_view);
	
	// connect to signals
	g_signal_connect (GTK_OBJECT(view_win->icon_view), "drag-data-received",
		G_CALLBACK(drag_received_cb), group);
	g_signal_connect (GTK_OBJECT(view_win->icon_view), "child-activated",
		G_CALLBACK(icon_view_child_activated_cb), group);
	g_signal_connect (GTK_OBJECT(view_win->icon_view), "child-selected",
		G_CALLBACK(icon_view_child_selected_cb), group);
	
	return view_win;
}

/// public methods

void
kiba_launcher_icon_view_toggle (KibaLauncherIconGroup *group)
{
	GList *l;
	
	if (KIBA_IS_ICON_VIEW_WIN(group->view_win)) {
		// toggle off
		gtk_widget_destroy (group->view_win->win);
	} else if (group->launcher->cfg_grouping) {
		// toggle on
		
		// destroy all other icon views, display only one
		for (l = group->launcher->groups; l != NULL; l = l->next) {
			KibaLauncherIconGroup *check_group = l->data;
			
			if (KIBA_IS_ICON_VIEW_WIN(check_group->view_win)) {
				gtk_widget_destroy (check_group->view_win->win);
			}
		}
		
		// create the icon view containing all group icons
		group->view_win = create_icon_view (group);
		gtk_widget_show (group->view_win->win);
		gdk_window_set_role (group->view_win->win->window, "kiba-launcher");
		gdk_window_set_keep_above (group->view_win->win->window, TRUE);
		icon_view_place (group);
		
		g_signal_connect(G_OBJECT (group->view_win->win), "destroy",
			G_CALLBACK(icon_view_win_destroy_cb), group);
		// connect to size allocation signal
		g_signal_connect_after(G_OBJECT (group->object), "size-allocate",
			G_CALLBACK(icon_view_size_allocate_cb), group);
		g_signal_connect_after(G_OBJECT (group->view_win->icon_view), "size-allocate",
			G_CALLBACK(icon_view_size_allocate_cb), group);
	}
}
