/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_LAUNCHER_H
#define __KIBA_LAUNCHER_H

#include <libintl.h>

#include <kiba-icon-view-win.h>
#include <kiba-desktop-icon.h>
#include <kiba-object.h>
#include <kiba-icon.h>
#include <kiba-plugin.h>

#define _(string)  dgettext (GETTEXT_PACKAGE, string)

G_BEGIN_DECLS

typedef struct _KibaLauncher KibaLauncher;

#define LAUNCHER_PLUGIN_NAME "launcher"
#define LAUNCHER_DIRECTORY_NAME "launcher"
#define ICON_LAUNCHER_KEY "launcher"
#define LAUNCHER_WINDOW_ROLE "kiba-launcher"

#define LAUNCHER_ICON DATADIR"/kiba-dock/icons/launcher.png"

typedef struct {
	KibaLauncher *launcher;
	
	KibaIconViewWin  *view_win;
	  
	// the group id (a Categorie or so)
	gchar *id;  
	// the dock object representing this group
	KibaObject *object;     
	// list with KibaDesktopIcon's from group,
	// the first one is the current active one
	GList *icons;          
	// num of icons added to the group
	gint count;      
}KibaLauncherIconGroup;

struct _KibaLauncher {
	KibaPlugin *plugin;
	
	gchar* launcher_directory;
	
	// list with all KibaLauncherIcons
	GList *icons;   
	// list with all KibaLauncherIconGroups
	GList *groups; 
	
	// preferences
	gboolean cfg_grouping;
	gfloat cfg_alpha;
	gfloat cfg_scale;
};

KibaLauncher*
kiba_launcher_new (GTypeModule *type_module,
	const gchar* launcher_directory);

void
kiba_launcher_add (KibaLauncher *launcher,
		KibaDesktopIcon *icon);

void
kiba_launcher_remove (KibaLauncher *launcher,
		KibaDesktopIcon *icon);

void
kiba_launcher_ungroup_icons (KibaLauncher *launcher);

void
kiba_launcher_group_icons (KibaLauncher *launcher);

KibaLauncherIconGroup*
kiba_launcher_find_icon_group (KibaLauncher *launcher,
		KibaDesktopIcon *desktop_icon);

void
kiba_launcher_select_icon(KibaLauncher *launcher,
		KibaLauncherIconGroup *group,
		KibaDesktopIcon *icon);

gboolean
kiba_launcher_drag_receive_cb (KibaLauncher *launcher,
		KibaLauncherIconGroup *group,
		GdkDragContext *drag_context,
		const gchar *drag_data);

void
kiba_launcher_destroy (KibaLauncher *launcher);



void
kiba_launcher_edid_icon (GtkWidget *widget,
		gpointer  data);

void
kiba_launcher_populate_dock (KibaDock *dock,
		gpointer data);

void
kiba_launcher_populate_dock_create (KibaDock *dock,
		gpointer data);

void
kiba_launcher_icon_view_toggle (KibaLauncherIconGroup *group);

void
icon_view_launcher_icon_added_cb(KibaLauncher *launcher,
		KibaDesktopIcon *desktop_icon,
		gpointer user_data);

void
icon_view_launcher_icon_removed_cb(KibaLauncher *launcher,
		KibaDesktopIcon *desktop_icon,
		gpointer user_data);

void
icon_view_launcher_icon_selected_cb(KibaLauncher *launcher,
		KibaDesktopIcon *desktop_icon,
		gpointer user_data);

#endif
