/*
 *  Copyright (C) 2009 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <daniel@orgizm.net>
*/

#ifndef __KIBA_REFLECTION_H__
#define __KIBA_REFLECTION_H__

#include <kiba-visual-feature.h>
#include <kiba-win.h>

G_BEGIN_DECLS

typedef struct _KibaSwitchPlugin KibaSwitchPlugin;
typedef struct _KibaSwitchPluginObject KibaSwitchPluginObject;

typedef struct {
	KibaWin *win;
	KibaVisualFeature *feature;
	KibaSwitchPlugin *switch_plugin;
	gboolean front;
}KibaSwitch3d;

struct _KibaSwitchPluginObject {
	KibaSwitchPlugin *switch_plugin;
	KibaObject *object;
	// scale factor
	float factor;
	// timeout id
	gint switch_id;
	// direction of switching
	gint switch_direction;
	gboolean switch_queued;
	float last_step;
	// saved rendering method
	void (* render_object_save) (KibaDrawableObject*, cairo_t*);
	// switch surface buffer
	CairoSurfaceData switch_surface;
	// fake 3d object features
	KibaVisualFeatureObject *front_feature_object;
	KibaVisualFeatureObject *back_feature_object;
};

struct _KibaSwitchPlugin {
	KibaWin *win;
	// list of KibaSwitchPluginObject's
	GList *objects;
	
	// fake 3d object features
	KibaSwitch3d *front_feature;
	KibaSwitch3d *back_feature;
	
	// settings
	float step;
	gboolean object3d;
	gint orientation;
};

void kiba_switch_3d_destroy(KibaSwitch3d *switch_3d);

KibaSwitch3d* kiba_switch_3d_new(
	KibaWin *win, KibaSwitchPlugin *switch_plugin,
	gboolean front);

G_END_DECLS

#endif
