/*
 *  Copyright (C) 2007-2009  Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#include "config.h"

#include <math.h>
#include <gtk/gtk.h>

#include <kiba.h>
#include <kiba-win.h>
#include <kiba-dock.h>
#include <kiba-object.h>
#include <kiba-plugin.h>
#include <kiba-prefs.h>
#include <kiba-debug.h>

#include "zoom.h"

/*** Signal Handler ***/

static void
__prefs_zoom_factor_cb (const char *option_id,
		gpointer  value,
		gpointer data)
{
	KibaZoom *zoom = data;
	zoom->cfg_magnitude = g_ascii_strtod((char*) value, NULL);
}

static void
__prefs_zoom_influence_cb (const char *option_id,
		gpointer value,
		gpointer data)
{
	KibaZoom *zoom = data;
	zoom->cfg_influence = g_ascii_strtod((char*) value, NULL);
}

/// Plugin Functions

static void
plugin_unload (KibaPlugin *plugin) {
	Kiba *kiba = kiba_get();
	KibaZoom *zoom;
	
	zoom = g_object_get_data (G_OBJECT(plugin), PLUGIN_NAME);
	
	parabolic_zoom_fini(zoom->current_mode);
	g_free (zoom);
}

static gboolean
plugin_load (KibaPlugin *plugin) {
	Kiba *kiba = kiba_get();
	KibaPrefsFile *file;
	KibaZoom *zoom;
	KibaWin *win;

	file = kiba_get_plugin_config_file(kiba, plugin);
	win = kiba_get_plugin_win(kiba, plugin);

	zoom = g_new0(KibaZoom, 1);
	zoom->plugin = plugin;
	zoom->win = win;
	zoom->cfg_magnitude = 1.0;
	zoom->cfg_influence = 0.05;
	zoom->current_mode = parabolic_zoom_init(zoom);

	kiba_prefs_add_callback (file, "zoom_factor",
		__prefs_zoom_factor_cb, zoom, TRUE);
	kiba_prefs_add_callback (file, "zoom_influence",
		__prefs_zoom_influence_cb, zoom, TRUE);

	g_object_set_data (G_OBJECT(plugin),
		PLUGIN_NAME, (gpointer) zoom);

	return TRUE;
}

static PrefMember zoom_options[] = {
    { "zoom_factor", "2.0", KIBA_DOUBLE },
    { "zoom_influence", "150.0", KIBA_DOUBLE }
};

static KibaPluginVtable vtable = {
	// description
	"Zoom Plugin for Kiba-Dock",
	NULL,

	// author
	"Daniel Bessler <danielb@kiba-dock.org>",
	// homepage
	"http://www.kiba-dock.org",

	// loadable flag
	KIBA_PLUGIN_MULTIPLE_LOADABLE,

	// load method
	plugin_load,
	// unload method
	plugin_unload,
	
	// plugin version
	MAJOR_VERSION,
	MINOR_VERSION
};

KIBA_INIT_PLUGIN (vtable, zoom_options, NULL, NULL, NULL);
